/*
 * CS356 Project 01 -- Elevator Simulator
 *
 * Elevator Class Specification
 */

#ifndef ELEVATOR_HPP
#define ELEVATOR_HPP

#include <iostream>
#include <list>
#include "direction.hpp"
#include "position.hpp"
#include "stop.hpp"

typedef std::list<Stop> StopList;

enum State { STATE_IDLE, STATE_UP, STATE_DOWN, STATE_WAIT, STATE_OPEN_DOOR, STATE_CLOSE_DOOR };
enum Event { EVT_IDLE, EVT_UP, EVT_DOWN, EVT_WAIT, EVT_OPEN_DOOR, EVT_CLOSE_DOOR };

class Elevator
{
	public:
		Elevator ();
		Elevator (int starting_floor);

		void stop_at (Stop &stop);
		float distance_from (Position& pos) const;
		void move ();
		bool is_idle () const;

	private:

		/* State Machine: Helper Functions */
		Event find_next_event () const;

		/* State Machine: Transition Functions */
		void transition_move_up ();
		void transition_move_down ();
		void transition_move_idle ();
		void transition_open_door ();
		void transition_close_door ();
		void transition_begin_wait ();
		void transition_continue_wait ();

		/* Analyze the list of stops */
		bool currently_at_stop () const;

		/* Elevator Status Variables */
		Direction	direction_;
		Position	position_;
		StopList	stops_;

		/* State Machine */
		State state_;
		int wait_;

		static const float ELEVATOR_STEP = 0.1;
};


#endif /* ELEVATOR_HPP */

/* vim: set ts=4 sts=4 sw=4 noexpandtab textwidth=112: */
